import React, { useState } from "react";
import { AxisLeft, AxisBottom } from '@vx/axis';
import { AxisRight, AxisTop } from '@vx/axis';
import { AreaStack } from '@vx/shape';
import { scaleLinear, scaleOrdinal } from '@vx/scale';
import { Group } from '@vx/group';
import { extent } from 'd3-array';
import { mathjax } from 'mathjax-full/js/mathjax';
import { TeX } from 'mathjax-full/js/input/tex';
import { SVG } from 'mathjax-full/js/output/svg';
import { browserAdaptor } from 'mathjax-full/js/adaptors/browserAdaptor';
import { RegisterHTMLHandler } from 'mathjax-full/js/handlers/html';
import { STATE } from 'mathjax-full/js/core/MathItem';
import { preprocessData, sumBase, sumStat, histogramify } from "../helpers/uhepp.js";

const getMaxBin =(uhepp) => {
  let max = 0;
  const n_bins = uhepp.bins.edges.length - 1
  for (let bin_i=0; bin_i < n_bins; bin_i++) {
    uhepp.stacks.forEach((stack, stack_index) => {
      let bottom = 0
      stack.content.forEach((stack_item, si_i) => {
        const y_value = sumBase(uhepp.yields, stack_item["yield"], bin_i)
        if (y_value > 0) { bottom += y_value; }
      })
      const whole_stack = stack.content.map(si => si["yield"]).flat()
      const stat = sumStat(uhepp.yields, whole_stack, bin_i)
      if (stat > 0) { bottom += stat; }
      max = Math.max(max, bottom)
    })
  }
  return max;
}

const Badge = ({brand, label, subtext}) => (
  <>
   <text y={0} x={0}>
     { brand &&
       <tspan className="uhepp-brand">{brand}</tspan>
     }
     { label && 
       <tspan dx={3} className="uhepp-label">{label}</tspan>
     }
   </text>
   { subtext && subtext.split("\n").map((s, i) => 
      <MixedText key={i} x={0} y={(i+1) * 16}>{s}</MixedText>
    )
   }
 </>
)

const Step = ({
  xScale,
  yScale,
  x,
  y,
  style={},
  defaultColorsCycle,
  id,
  highlightedBin=false,
  onMouseOver=()=>null,
  onMouseOut=()=>null,
}) => {
  const points = Array(x.length).fill(0).map(
    (_, i) => [xScale(x[i]), yScale(y[i])]
  )

  // const str_token = points.map(([x, y], i) => `${i ? "L" : "M"}${x} ${y}`)
  // const data = [...str_token, "Z"].join(" ")
  //const str_token = points.map(([x, y], i) => `${x},${y}`)
  //const data = [...str_token].join(" ")

  return <g>
    { Array(x.length - 1).fill(0).map(
     (_, i) => <line x1={points[i][0]} y1={points[i][1]}
                     x2={points[i + 1][0]} y2={points[i + 1][1]}
                     key={i}
                     strokeWidth={style.linewidth || 1.5}
                     stroke={style.color || defaultColorsCycle.get(id)}
                     opacity={i / 2 == highlightedBin ? 0.5 : 1}
                     />
    )}
  </g>
}


const computeEquidistent = (edges) => {
  let width = false
  for (let i = 0; i < edges.length - 1; i++) {
    const this_width = edges[i + 1] - edges[i]
    if (width === false || this_width == width) {
      width = this_width
    } else {
      return false
    }
  }
  return width
}

const Bar = ({
  xScale,
  yScale,
  leftEdge,
  rightEdge,
  bottom,
  y,
  id,
  style={},
  defaultColorsCycle,
  highlighted=false,
  onMouseOver=()=>null,
  onMouseOut=()=>null,
}) => (
  <rect x={xScale(leftEdge)}
        width={xScale(rightEdge) - xScale(leftEdge)}
        y={yScale(bottom + y)}
        height={yScale(bottom) - yScale(bottom + y)}
        fill={style.color || defaultColorsCycle.get(id)}
        opacity={highlighted ? 0.5 : 1.0}
        onMouseOver={() => onMouseOver()}
        onMouseOut={() => onMouseOut()} />
)

const Point = ({
  xScale,
  yScale,
  x,
  y,
  xErr=0,
  yErr=0,
  style={},
  highlighted=false,
  onMouseOver=()=>null,
  onMouseOut=()=>null,
}) => (
  <g onMouseOver={() => onMouseOver()}
     onMouseOut={() => onMouseOut()}>
    <circle cx={xScale(x)}
            cy={yScale(y)}
            fill={style.color || "#000"}
            opacity={highlighted ? 0.8 : 1.0}
            r="3" />
    <line y1={yScale(y)}
          y2={yScale(y)}
          stroke={style.color || "#000"}
          strokeWidth="2"
          strokeOpacity={highlighted ? 0.8 : 1.0}
          x1={xScale(x - xErr)}
          x2={xScale(x + xErr)} />
    <line y1={yScale(y + yErr)}
          y2={yScale(y - yErr)}
          strokeWidth="2"
          stroke={style.color || "#000"}
          strokeOpacity={highlighted ? 0.8 : 1.0}
          x1={xScale(x)}
          x2={xScale(x)} />
  </g>
)


class ColorCycle {
  constructor(colors) {
    this.colors = colors
    this.index = 0
    this.cache = {}
  }

  length() {
    return this.colors.length
  }

  get(id) {
    if (!this.cache.hasOwnProperty(id)) {
      const item = this.colors[this.index]
      this.index = (this.index + 1) % this.colors.length
      this.cache[id] = item
    }
    return this.cache[id]
  }
}

const Ratio = ({
  uhepp,
  ratioScale,
  xScale,
  defaultColorsCycle,
  highlightedBin,
  onMouseOverBin,
  children
}) => {
  const edges = uhepp.bins.rebin || uhepp.bins.edges
  const n_bins = edges.length - 1
  let objects = []

  uhepp.ratio.forEach((ratio_item, ri_i) => {

    if (ratio_item.type == "step") {
        const bin_indices = Array(n_bins).fill(0).map((_, i) => i)

        const numerator = bin_indices.map(i => sumBase(uhepp.yields, ratio_item.numerator, i + 1))
        const denominator = bin_indices.map(i => sumBase(uhepp.yields, ratio_item.denominator, i + 1))

        const stat_num = bin_indices.map(i => sumStat(uhepp.yields, ratio_item.numerator, i + 1))
        const stat_den = bin_indices.map(i => sumStat(uhepp.yields, ratio_item.denominator, i + 1))


        const y_values = bin_indices.map(i => numerator[i] / denominator[i])
        const [new_x, new_y] = histogramify(edges, y_values)

        const stat = bin_indices.map(i => Math.sqrt(stat_num[i]**2 / denominator[i]**2
                               + numerator[i]**2 / denominator[i]**4 * stat_den[i]**2))

        objects.push(<Step x={new_x}
                           y={new_y}
                           xScale={xScale}
                           yScale={ratioScale}
                           style={ratio_item.style}
                           defaultColorsCycle={defaultColorsCycle}
                           id={`${ri_i}`}
                           key={`${ri_i}`}
                           highlightedBin={highlightedBin}
                           onMouseOver={() => onMouseOverBin(bin_i)}
                           onMouseOut={() => onMouseOverBin(null)} />)

    } else if (ratio_item.type == "points") {
      for (let bin_i=0; bin_i < n_bins; bin_i++) {
        const numerator = sumBase(uhepp.yields, ratio_item.numerator, bin_i + 1)
        const denominator = sumBase(uhepp.yields, ratio_item.denominator, bin_i + 1)

        const stat_num = sumStat(uhepp.yields, ratio_item.numerator, bin_i + 1)
        const stat_den = sumStat(uhepp.yields, ratio_item.denominator, bin_i + 1)

        const y_value = numerator / denominator
        const stat = Math.sqrt(stat_num**2 / denominator**2
                               + numerator**2 / denominator**4 * stat_den**2)
        const bin_center = (edges[bin_i + 1] + edges[bin_i]) / 2
        const width = edges[bin_i + 1] - edges[bin_i]

        if (isFinite(stat) && isFinite(y_value)) {
          objects.push(<Point x={bin_center}
                              y={y_value}
                              xErr={width / 2}
                              yErr={stat}
                              xScale={xScale}
                              yScale={ratioScale}
                              key={`key-${ri_i}-${bin_i}`}
                              highlighted={highlightedBin == bin_i}
                              onMouseOver={() => onMouseOverBin(bin_i)}
                              style={ratio_item.style} />)
                              
        }
      }
    }

    for (let bin_i=0; bin_i < n_bins; bin_i++) {
      const denominator = sumBase(uhepp.yields, ratio_item.denominator, bin_i + 1)
      const stat_den = sumStat(uhepp.yields, ratio_item.denominator, bin_i + 1)
      const bin_center = (edges[bin_i + 1] + edges[bin_i]) / 2

      const rel_stat = stat_den / denominator
      if (isFinite(rel_stat)) {
        objects.splice(0, 0, <rect
          key={`rect-uncert-${ri_i}-${bin_i}`}
          x={xScale(edges[bin_i - 1])}
          width={xScale(edges[bin_i + 1]) - xScale(edges[bin_i])}
          y={ratioScale(1 + rel_stat)}
          height={ratioScale(1 - rel_stat) - ratioScale(1 + rel_stat)}
          fill="url(#errorBand)"
          onMouseOver={() => onMouseOverBin(bin_i)}
          onMouseOut={() => onMouseOverBin(null)} />)
      }
    }
  })

  return objects
}

const Histogram = ({
  uhepp,
  yScale,
  xScale,
  defaultColorsCycle,
  highlightedBin,
  onMouseOverBin,
  children
}) => {

  const edges = uhepp.bins.rebin || uhepp.bins.edges
  const n_bins = edges.length - 1
  let objects = []

  // for (let bin_i=1; bin_i < n_bins; bin_i++) {
    uhepp.stacks.forEach((stack, stack_index) => {
      let bottom = Array(n_bins).fill(0)
      stack.content.forEach((stack_item, si_i) => {
        if (stack.type == "stepfilled") {
          for (let bin_i=0; bin_i < n_bins; bin_i++) {
            const y_value = sumBase(uhepp.yields, stack_item["yield"], bin_i + 1)
            objects.push(<Bar leftEdge={edges[bin_i]}
                              rightEdge={edges[bin_i + 1]}
                              bottom={bottom[bin_i]}
                              y={y_value}
                              xScale={xScale}
                              yScale={yScale}
                              style={stack_item.style}
                              highlighted={highlightedBin == bin_i}
                              defaultColorsCycle={defaultColorsCycle}
                              id={`${stack_index}-${si_i}`}
                              key={`key-${stack_index}-${si_i}-${bin_i}`}
                              onMouseOver={() => onMouseOverBin(bin_i)}
                              onMouseOut={() => onMouseOverBin(null)} />)
            if (y_value > 0) {
              bottom[bin_i] += y_value
            }
          }
        } else if (stack.type == "step") {
          const bin_indices = Array(n_bins).fill(0).map((_, i) => i)
          const y_values = bin_indices.map(i => sumBase(uhepp.yields, stack_item["yield"], i + 1))

          const [new_x, new_y] = histogramify(edges, y_values)

          objects.push(<Step x={new_x}
                             y={new_y}
                             xScale={xScale}
                             yScale={yScale}
                             style={stack_item.style}
                             defaultColorsCycle={defaultColorsCycle}
                             id={`${stack_index}-${si_i}`}
                             key={`${stack_index}-${si_i}`}
                             highlightedBin={highlightedBin}
                             onMouseOver={() => onMouseOverBin(bin_i)}
                             onMouseOut={() => onMouseOverBin(null)} />)

            y_values.forEach((y_value, bin_i) => {
              if (y_value > 0) {
                bottom[bin_i] += y_value
              }
            })
        } else if (stack.type == "points") {
          for (let bin_i=0; bin_i < n_bins; bin_i++) {
            const y_value = sumBase(uhepp.yields, stack_item["yield"], bin_i + 1)
            const stat = sumStat(uhepp.yields, stack_item["yield"], bin_i + 1)
            const bin_center = (edges[bin_i + 1] + edges[bin_i]) / 2
            const width = edges[bin_i + 1] - edges[bin_i]

            if (stat && y_value) {
              objects.push(<Point x={bin_center}
                                  y={bottom[bin_i] + y_value}
                                  xErr={width / 2}
                                  yErr={stat}
                                  xScale={xScale}
                                  yScale={yScale}
                                  key={`key-${stack_index}-${si_i}-${bin_i}`}
                                  highlighted={highlightedBin == bin_i}
                                  onMouseOver={() => onMouseOverBin(bin_i)}
                                  style={stack_item.style} />)
              if (y_value > 0) {
                bottom[bin_i] += y_value
              }
                                  
            }
          }
        }
      })

			if (stack.type == "stepfilled" || stack.type == "step") {
        for (let bin_i=0; bin_i < n_bins; bin_i++) {
          const whole_stack = stack.content.map(si => si["yield"]).flat()
          const stat = sumStat(uhepp.yields, whole_stack, bin_i + 1)
          objects.push(<rect
            key={`rect-${stack_index}-uncert-${bin_i}`}
            x={xScale(edges[bin_i])}
            width={xScale(edges[bin_i + 1]) - xScale(edges[bin_i])}
            y={yScale(bottom[bin_i] + stat)}
            fill="url(#errorBand)"
            height={yScale(bottom[bin_i] - stat) - yScale(bottom[bin_i] + stat)}
            onMouseOver={() => onMouseOverBin(bin_i)}
            onMouseOut={() => onMouseOverBin(null)} />)
        }
      }
    })
  //}

  return objects
}

const adaptor = browserAdaptor();
RegisterHTMLHandler(adaptor);
function onError(math) {
  const {root, typesetRoot} = math;
  if (root.toString().substr(0,14) === 'math([merror([') {
    const merror = root.childNodes[0].childNodes[0];
    const text = merror.attributes.get('data-mjx-error') || merror.childNodes[0].childNodes[0].getText();
    adaptor.setAttribute(typesetRoot, 'data-mjx-error', text);
  }
}
function updateCSS(nodeID, text) {
  let styleNode = document.getElementById(nodeID);
  if (styleNode === null) {
    styleNode = document.createElement('style');
    styleNode.setAttribute('id', nodeID);
    document.head.appendChild(styleNode);
  }
  styleNode.innerHTML = text;
}

const tex = new TeX({packages: ['base', 'ams']});
const svg = new SVG({fontCache: 'none'});
const markErrors = [STATE.TYPESET + 1, null, onError];
const tex_html = mathjax.document('', {
  InputJax: tex,
  OutputJax: svg,
  renderActions: {
    markErrors,
  }
});

const length_metrix = (str) => {
  var short_count = (str.match(/[,.tli1: ]/g) || []).length;
  var long_count = (str.match(/[MWXZ]/g) || []).length;
  return str.length + 0.2 * long_count - 0.5 * short_count
}

const EmbeddedMathJax = ({src, posX, posY, align="left"}) => {
  let html = tex_html;
  let node = document.createElement("div");
  const math = src.trim();
  const metrics = svg.getMetricsFor(node, true);
  const outerHTML =  adaptor.outerHTML(html.convert(math, {
          display: true,
          ... metrics
  })); 
  html.updateDocument();
  updateCSS('MATHJAX-SVG-STYLESHEET', svg.cssStyles.cssText);
	node.innerHTML = outerHTML;
	let rendered_svg = node.getElementsByTagName("svg")[0];
	const scale = 6
	const width = rendered_svg.width.baseVal.value * scale
	const transX = rendered_svg.width.baseVal.value * scale / rendered_svg.viewBox.baseVal.width
	const transY = rendered_svg.height.baseVal.value * scale / rendered_svg.viewBox.baseVal.height
  let alignTransX = 0
  if (align == "center") {
    alignTransX = -width / 2
  } else if (align == "right") {
    alignTransX = -width
  }

  return <g
            transform={`matrix(${transX} 0 0 ${transY} ${posX + alignTransX} ${posY})`}
					  dangerouslySetInnerHTML={{__html: rendered_svg.innerHTML}}></g>
}

const MixedText = ({children, x=0, y=0, align="left", opacity=1}) => {
  if (children.join) {
    children = children.join("")
  }
  const embedded = `\\textsf{${children}}`
  const switched = embedded.replaceAll(/\$([^$]*)\$/g, "}$1\\textsf{")
  const cleared = switched.replaceAll("\\textsf{}", "")

  return <g opacity={1 * opacity}>
    <EmbeddedMathJax posX={x} posY={y} src={cleared} align={align} />
  </g>
}

const Legend = ({
  post_uhepp,
  setShowTotal,
  defaultColorsCycle,
  showTotal,
  highlightedBin
}) => {
  const lineSkip = 16;
  const mathSkip = 120;
  let i = 0
  let legend = []
  post_uhepp.stacks.map((stack, stack_index) => {
      stack.content.map((stack_item, si_i) => {
        if (stack.type == "points") {
            legend.push(<Point x={10}
                                y={(i + 1) * lineSkip - 5}
                                xErr={10}
                                yErr={5}
                                xScale={(x) => x}
                                yScale={(y) => y}
                                key={`legend-${i}`}
                                onMouseOver={() => setShowTotal(true)}
                                onMouseOut={() => setShowTotal(false)}
                                style={stack_item.style} />)
        } else if (stack.type == "step") {
          legend.push(<Step x={[0, 20]}
                            y={[5, 5]}
                            y={Array(2).fill((i + 1) * lineSkip - 5)}
                            xScale={(x) => x}
                            yScale={(y) => y}
                            style={Object.assign({}, stack_item.style, {linewidth: 2})}
                            onMouseOver={() => setShowTotal(true)}
                            onMouseOut={() => setShowTotal(false)}
                            highlightedBin={null}
                            defaultColorsCycle={defaultColorsCycle}
                            id={`${stack_index}-${si_i}`}
                            key={`legend-${stack_index}-${si_i}`} />)
        } else {
          legend.push(<Bar leftEdge={0}
                            rightEdge={20}
                            bottom={(i + 1) * lineSkip}
                            y={-10}
                            xScale={(x) => x}
                            yScale={(y) => y}
                            style={stack_item.style}
                            onMouseOver={() => setShowTotal(true)}
                            onMouseOut={() => setShowTotal(false)}
                            defaultColorsCycle={defaultColorsCycle}
                            id={`${stack_index}-${si_i}`}
                            key={`legend-${stack_index}-${si_i}`} />)
        }
        let showLabel = false
        if (highlightedBin != null) {
          const y_value = sumBase(post_uhepp.yields, stack_item["yield"], highlightedBin + 1)
          const stat = sumStat(post_uhepp.yields, stack_item["yield"], highlightedBin + 1)

          const label = y_value.toFixed(1) + " ± " + stat.toFixed(1)
          legend.push(<text fontSize={12} textAnchor="end" x={mathSkip} y={(i + 1) * lineSkip}
                key={`legend-text-${i}`}>{label}</text>)
        } else if (showTotal) {
          const y_value = sumBase(post_uhepp.yields, stack_item["yield"])
          const stat = sumStat(post_uhepp.yields, stack_item["yield"])

          const label = y_value.toFixed(1) + " ± " + stat.toFixed(1)
          legend.push(<text fontSize={12} textAnchor="end" x={mathSkip} y={(i + 1) * lineSkip}
                key={`legend-text-${i}`}>{label}</text>)

        } else {
          showLabel = true
        }
        const label = stack_item.label
        legend.push(
          <MixedText x={30} y={(i + 1) * lineSkip} opacity={showLabel}
                key={`legend-label-${i}`}>{label}</MixedText>)
        i++;
      })
  })

  return legend
}

const UheppHist = ({width, height, uhepp}) => {

  const [highlightedBin, setHighlightedBin] = useState(null);
  const [showTotal, setShowTotal] = useState(false);

  const defaultColorsCycle = new ColorCycle([
    "#1f77b4",
    "#ff7f0e",
    "#2ca02c",
    "#d62728",
    "#9467bd",
    "#8c564b",
    "#e377c2",
    "#7f7f7f",
    "#bcbd22",
    "#17becf",
    "#1f77b4",
  ])

  const post_uhepp = Object.assign({}, uhepp, {
      yields: preprocessData({
        yields: uhepp.yields,
        old_edges: uhepp.bins.edges,
        new_edges: uhepp.bins.rebin, 
        include_underflow: uhepp.bins.include_underflow, 
        include_overflow: uhepp.bins.include_overflow, 
      })
  })

	const margin = {
		top: 10,
		bottom: 60,
		left: 80,
		right: 80,
    sep: 3,
	}

	const xMax = width - margin.left - margin.right
	let yMax = height - margin.top - margin.bottom 
  let ratioMax = 0 
  const hasRatio = (uhepp.ratio && uhepp.ratio.length)
  if (hasRatio) {
    const ratioFraction = (uhepp.layout && uhepp.layout.ratio_fraction) || 0.25
	  yMax = (height - margin.top - margin.bottom - margin.sep) * (1 - ratioFraction)
	  ratioMax = (height - margin.top - margin.bottom - margin.sep) * (ratioFraction)
  }

	const xScale = scaleLinear({
		range: [0, xMax],
		domain: extent(uhepp.bins.edges),
	})
	const yScale = scaleLinear({
		range: [yMax, 0],
		domain: [0, getMaxBin(post_uhepp) * 1.5],
	})
	const ratioScale = scaleLinear({
		range: [ratioMax, 0],
		domain: [
      (uhepp.ratio_axis && uhepp.ratio_axis.min) || 0.5,
      (uhepp.ratio_axis && uhepp.ratio_axis.max) || 1.5,
    ],
	})



  const equidistent = computeEquidistent(uhepp.bins.rebin || uhepp.bins.edges)

	return (
  <div style={{width: '100%', margin: '0 auto'}}>
			<svg viewBox={`0 0 ${width} ${height}`}
           onMouseOut={() => setHighlightedBin(null)} >
        <style>
        rect {'{shape-rendering: crispEdges }'}
        .uhepp-brand {'{font-style: italic; font-weight: bold}'}
        .uhepp-brand, .uhepp-label {'{font-size: 20px}'}
        </style>
        <defs>
          <clipPath id="main-clip">
            <rect x={0} y={0} width={xMax} height={yMax} />
          </clipPath>
          { hasRatio && 
            <clipPath id="ratio-clip">
              <rect x={0} y={0} width={xMax} height={ratioMax} />
            </clipPath>
          }
          <pattern id="errorBand" patternUnits="userSpaceOnUse" width="6" height="6">
            <path d="M-1,1 l1,-1 M0,6 l6,-6 M5,7 l2,-2" style={{stroke: "#666",  strokeWidth: 1}} />
          </pattern>
        </defs>
				<Group top={margin.top} left={margin.left}>
          <g clipPath="url(#main-clip)">
            <Histogram
               uhepp={post_uhepp}
               highlightedBin={highlightedBin}
               onMouseOverBin={binIndex => setHighlightedBin(binIndex)}
               defaultColorsCycle={defaultColorsCycle}
               xScale={xScale}
               yScale={yScale}
             />
         </g>

        <Group top={26} left={13}>
          <Badge brand={uhepp.badge.brand} label={uhepp.badge.label} subtext={uhepp.badge.subtext} />
        </Group>

        <Group top={5} left={xMax * 0.65}>
          <Legend post_uhepp={post_uhepp}
                  setShowTotal={setShowTotal}
                  showTotal={showTotal}
                  defaultColorsCycle={defaultColorsCycle}
                  highlightedBin={highlightedBin} />
        </Group>

		 		<AxisBottom
						scale={xScale}
						top={yMax}
						stroke={'#1b1a1e'}
						tickTextFill={'#1b1a1e'}
						tickTransform="translate(0, -8)"
					  tickComponent={hasRatio ? (props) => <></> : undefined}
					/>
		 		<AxisBottom
						scale={xScale}
						top={yMax}
            numTicks={50}
						stroke={'#1b1a1e'}
            tickLength={4}
						tickTransform="translate(0, -4)"
					  tickComponent={(props) => <></>}
					/>
        { !hasRatio && 
          <MixedText y={yMax + 30} x={xMax - 10} align="right">
            {uhepp.variable.name ? uhepp.variable.name + ' ' : ''}
            {uhepp.variable.symbol}
            {uhepp.variable.unit ? ' / ' + uhepp.variable.unit : ''}
          </MixedText>
        }

				<AxisLeft
						scale={yScale}
						top={0}
						left={0}
						stroke={'#1b1a1e'}
						tickTextFill={'#1b1a1e'}
						tickTransform="translate(8, 0)"
					/>
        <g transform={`matrix(0 -1 1 0 ${-margin.left + 20} 10)`}>
          <MixedText y={0} x={0} align="right">
            Events
            { equidistent ? ' / ' + equidistent + (uhepp.variable.unit ? " " + uhepp.variable.unit : '') : ''}
          </MixedText>
        </g>

				<AxisLeft
						scale={yScale}
						top={0}
						left={0}
						stroke={'#1b1a1e'}
            numTicks={50}
						tickTransform="translate(4, 0)"
            tickLength={4}
					  tickComponent={(props) => <></>}
					/>
				<AxisRight
						scale={yScale}
						top={0}
						left={xMax}
						stroke={'#1b1a1e'}
					  tickComponent={(props) => <></>}
						tickTransform="translate(-8, 0)"
					/>
				<AxisRight
						scale={yScale}
						top={0}
						left={xMax}
						stroke={'#1b1a1e'}
            numTicks={50}
					  tickComponent={(props) => <></>}
            tickLength={4}
						tickTransform="translate(-4, 0)"
					/>
					<AxisTop
						scale={xScale}
						top={0}
						stroke={'#1b1a1e'}
					  tickComponent={(props) => <></>}
						tickTransform="translate(0, 8)"
					/>
					<AxisTop
						scale={xScale}
						top={0}
						stroke={'#1b1a1e'}
            numTicks={50}
					  tickComponent={(props) => <></>}
            tickLength={4}
						tickTransform="translate(0, 4)"
					/>
				</Group>

		<Group top={margin.top + margin.sep + yMax} left={margin.left}>
         <g clipPath="url(#ratio-clip)">
           <Ratio
              uhepp={post_uhepp}
              highlightedBin={highlightedBin}
              onMouseOverBin={binIndex => setHighlightedBin(binIndex)}
              defaultColorsCycle={defaultColorsCycle}
              xScale={xScale}
              ratioScale={ratioScale}
            />
        </g>
		 		<AxisBottom
						scale={xScale}
						top={ratioMax}
						stroke={'#1b1a1e'}
						tickTextFill={'#1b1a1e'}
						tickTransform="translate(0, -8)"
					/>
		 		<AxisBottom
						scale={xScale}
						top={ratioMax}
            numTicks={50}
						stroke={'#1b1a1e'}
            tickLength={4}
						tickTransform="translate(0, -4)"
					  tickComponent={(props) => <></>}
					/>
        <MixedText y={ratioMax + 30} x={xMax - 10} align="right">
          {uhepp.variable.name ? uhepp.variable.name + ' ' : ''}
          {uhepp.variable.symbol}
          {uhepp.variable.unit ? ' / ' + uhepp.variable.unit : ''}
        </MixedText>

				<AxisLeft
						scale={ratioScale}
						top={0}
						left={0}
						stroke={'#1b1a1e'}
						tickTextFill={'#1b1a1e'}
            numTicks={5}
						tickTransform="translate(8, 0)"
					/>
        { uhepp.ratio_axis.label && 
          <g transform={`matrix(0 -1 1 0 ${-margin.left + 20} ${ratioMax / 2})`}>
            <MixedText y={0} x={0} align="center">
              { uhepp.ratio_axis.label }
            </MixedText>
          </g>
        }

				<AxisLeft
						scale={ratioScale}
						top={0}
						left={0}
						stroke={'#1b1a1e'}
            numTicks={20}
						tickTransform="translate(4, 0)"
            tickLength={4}
					  tickComponent={(props) => <></>}
					/>
				<AxisRight
						scale={ratioScale}
						top={0}
						left={xMax}
            numTicks={5}
						stroke={'#1b1a1e'}
					  tickComponent={(props) => <></>}
						tickTransform="translate(-8, 0)"
					/>
				<AxisRight
						scale={ratioScale}
						top={0}
						left={xMax}
						stroke={'#1b1a1e'}
            numTicks={20}
					  tickComponent={(props) => <></>}
            tickLength={4}
						tickTransform="translate(-4, 0)"
					/>
					<AxisTop
						scale={xScale}
						top={0}
						stroke={'#1b1a1e'}
					  tickComponent={(props) => <></>}
						tickTransform="translate(0, 8)"
					/>
					<AxisTop
						scale={xScale}
						top={0}
						stroke={'#1b1a1e'}
            numTicks={50}
					  tickComponent={(props) => <></>}
            tickLength={4}
						tickTransform="translate(0, 4)"
					/>
        </Group>
			</svg>
  </div>
	)
}
export default UheppHist;
